/*
 * Copyright 1991, 1992, 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
/* 
 *	objstat -
 *		Print info about objects.
 *
 *			Paul Haeberli - 1990
 */
#include "stdio.h"
#include "math.h"
#include "vect.h"
#include "sgiobj.h"
#include "spin.h"

#define OBJ_OLDBIN	12

main(argc,argv)
int argc;
char **argv;
{
    sgiobj *obj;
    int i;

    if(argc<2) {
	fprintf(stderr,"usage: objstat objects . . .\n");
	exit(1);
    }
    printf("  objtype ");
    printf("   nploys ");
    printf("  npoints ");
    printf("  name\n");
    for(i=1; i<argc; i++) 
	statobj(argv[i]);
}

statobj(name)
char *name;
{
    FILE *inf;
    sgiobj *obj;
    int objtype, nlongs;
    int magic, npolys, npoints;

    inf = fopen(name,"r");
    if(!inf) {
        fprintf(stderr,"statobj: can't open input file %s\n",name);
        exit(1);
    }
    fread(&magic,sizeof(long),1,inf);
    if(magic == FASTMAGIC) {
	fread(&npoints,sizeof(long),1,inf);
	objtype = OBJ_OLDBIN;
	nlongs = npoints*PNTLONGS;
    } else if(magic == SOMAGIC) {
	fread(&objtype,sizeof(long),1,inf);
	fread(&nlongs,sizeof(long),1,inf);
    } else {
        fprintf(stderr,"statobj: bad magic %d\n",magic);
        exit(1);
    }
    fclose(inf);
    switch(objtype) {
	case OBJ_OLDBIN:
	    printf("   SPIN   ");
	    printf("  %6d  ",(nlongs/PNTLONGS)/4);
	    printf("  %6d  ",nlongs/PNTLONGS);
	    break;
	case OBJ_QUADLIST:
	    printf(" QUADLIST ");
	    printf("  %6d  ",(nlongs/PNTLONGS)/4);
	    printf("  %6d  ",nlongs/PNTLONGS);
	    break;
	case OBJ_TRILIST:
	    printf(" TRILIST  ");
	    printf("  %6d  ",(nlongs/PNTLONGS)/3);
	    printf("  %6d  ",nlongs/PNTLONGS);
	    break;
	case OBJ_TRIMESH:
	    printf(" TRIMESH  ");
	    obj = readsgiobj(name);
	    objstats(obj,&npolys,&npoints);
	    freesgiobj(obj);
	    printf("  %6d  ",npolys);
	    printf("  %6d  ",npoints);
	    break;
	default:
	    printf("  WIERD   ");
	    printf("  ????    ");
	    printf("  ????    ");
	    break;
    }
    printf("  %s\n",name);
}

objstats(obj,npolys,npoints)
sgiobj *obj;
int *npolys, *npoints;
{
    long *data;

    data = obj->data;
    if(obj->objtype == OBJ_QUADLIST) {
	*npolys = (obj->nlongs/PNTLONGS)/4;
	*npoints = 4 * (*npolys);
    } else if(obj->objtype == OBJ_TRILIST) {
	*npolys = (obj->nlongs/PNTLONGS)/3;
	*npoints = 3 * (*npolys);
    } else if(obj->objtype == OBJ_TRIMESH) {
	*npolys = objpolys(obj);
	*npoints = (*data++)/PNTLONGS;
    } else {
	fprintf(stderr,"objstats: bad object type %d\n",obj->objtype);
	exit(1);
    }
}
