/*
 * Copyright 1991, 1992, 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
/*
 *	tldraw -
 *		Draw a triangle list.
 *
 *			Paul Haeberli - 1990
 */
#include "stdio.h"
#include "gl.h"
#include "device.h"
#include "trilist.h"

float fgetmousex();
float fgetmousey();

main(argc,argv)
int argc;
char **argv;
{
    int dofill, dozbuffer;
    short val;
    float x, y;
    int rendermode;

    if(argc<2) {
	fprintf(stderr, "usage: tldraw input.tl [-o name.rgb]\n");
	exit(1);
    }
    if(argc==4) 
	rendermode = 1;	
    else 
	rendermode = 0;	
    if(rendermode) {
	foreground();
	prefposition(0,899,0,599);
    } else
	keepaspect(3,2);
    winopen("tldraw");
    RGBmode();
    gconfig();
    dozbuffer = 1;
    dofill = 1;
    drawtrilist(argv[1],dofill,dozbuffer);
    if(rendermode) {
	 savergbwindow(argv[3]);
	 exit(0);
    }
    qdevice(LEFTMOUSE);
    qdevice(MIDDLEMOUSE);
    qdevice(WKEY);
    while(1) {
	switch(qread(&val)){
	    case REDRAW:
		drawtrilist(argv[1],dofill,dozbuffer);
		break;
	    case LEFTMOUSE:
		if(val) {
		    dozbuffer = 1-dozbuffer;
		    drawtrilist(argv[1],dofill,dozbuffer);
		}
		break;
	    case MIDDLEMOUSE:
		if(val) {
		    dofill = 1-dofill;
		    drawtrilist(argv[1],dofill,dozbuffer);
		}
		break;
	    case WKEY:
		if(val) {
		    x = 2.0*(fgetmousex()-0.5);
		    y = 2.0*(fgetmousey()-0.5);
		    printf("cur pos %f %f\n",x,y);
		}
		break;
	}
    }
}

drawtrilist(name,dofill,doz)
char *name;
int dofill, doz;
{
    int i, ntri;
    trilisttri tri;

    if(doz)
       wintitle("tldraw - zbuffer");
    else
       wintitle("tldraw - painting");
    ntri = tlopen(name);
    fprintf(stderr, "ntri: %d\n",ntri);
    zbuffer(doz);
    reshapeviewport();
    ortho(-1.0,1.0,-1.0,1.0,-2.0,2.0);
    zclear();
    cpack(0x00ffffff);
    clear();
    for(i=0; i<ntri; i++) {
	tlread(&tri);
	tridraw(&tri,dofill);
    }
    tlclose();
}

#define ISTRANS(c) (((c)>>24) == 0x00)

tridraw(tri,dofill)
trilisttri *tri;
{
    if(ISTRANS(tri->c0))
	dofill = 0;
    if(dofill) {
	bgnpolygon();
	cpack(tri->c0);
	v3f(&tri->x0);
	cpack(tri->c1);
	v3f(&tri->x1);
	cpack(tri->c2);
	v3f(&tri->x2);
	endpolygon();
    } else {
	cpack(0x00000000);
	bgnclosedline();
	v3f(&tri->x0);
	v3f(&tri->x1);
	v3f(&tri->x2);
	endclosedline();
    }
}

triprint(tri)
trilisttri *tri;
{
    printf("v0: %f %f %f c0: %08x\n",tri->x0,tri->y0,tri->z0,tri->c0);
    printf("v1: %f %f %f c1: %08x\n",tri->x1,tri->y1,tri->z1,tri->c1);
    printf("v2: %f %f %f c2: %08x\n\n",tri->x2,tri->y2,tri->z2,tri->c2);
}
