/*
 * Copyright 1991, 1992, 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
/* 
 *	tllint -
 *		Merge coordinates that are almost identical.
 *
 *			Paul Haeberli - 1991
 */
#include "math.h"
#include "stdio.h"
#include "vect.h"
#include "trilist.h"

#define TOLERANCE	(0.001)

main(argc,argv)
int argc;
char **argv;
{
    if(argc<3) {
	fprintf(stderr,"tllint: in.tl out.tl\n");
	exit(1);
    }
    tllint(argv[1],argv[2]);
    exit(0);
}

#define TABSIZE		512

typedef struct vert {
    struct vert *next;
    float pos[3];
} vert;

float ymin, ymax, deltay;
vert *table[TABSIZE];

inithash()
{
    int i;

    for(i=0; i<TABSIZE; i++) 
	table[i] = 0;
}

tllint(inname,outname)
char *inname, *outname;
{
    trilisttri tri;
    FILE *outf;
    int i, ntri;
    short magic;

/* find the y range */
    ntri = tlopen(inname);
    ymin = 10000.0;
    ymax = -10000.0;
    for(i=0; i<ntri; i++) {
	tlread(&tri);
	if(ymin>tri.y0)
	    ymin=tri.y0;
	if(ymin>tri.y1)
	    ymin=tri.y1;
	if(ymin>tri.y2)
	    ymin=tri.y2;
	if(ymax<tri.y0)
	    ymax=tri.y0;
	if(ymax<tri.y1)
	    ymax=tri.y1;
	if(ymax<tri.y2)
	    ymax=tri.y2;
    }
    tlclose();
    deltay = ymax-ymin;

    outf = fopen(outname,"w");
    magic = TLMAGIC;
    fwrite(&magic,sizeof(short),1,outf);
    ntri = tlopen(inname);
    for(i=0; i<ntri; i++) {
	tlread(&tri);
	fixpos(&tri.x0);
	fixpos(&tri.x1);
	fixpos(&tri.x2);
	if(!possame(&tri.x0,&tri.x1) && 
		   !possame(&tri.x1,&tri.x2) && !possame(&tri.x2,&tri.x0))
	    fwrite(&tri,sizeof(trilisttri),1,outf);
    }
    tlclose();

    fclose(outf);
}

fixpos(p)
float p[3];
{
    int t, tabpos1, tabpos2;
    vert *v;

    tabpos1 = (TABSIZE-1)*(p[1]-ymin-TOLERANCE)/deltay;
    if(tabpos1<0)
       	tabpos1 = 0;
    tabpos2 = (TABSIZE-1)*(p[1]-ymin+TOLERANCE)/deltay;
    if(tabpos2>=TABSIZE)
       	tabpos2 = TABSIZE-1;
    for(t=tabpos1; t<=tabpos2; t++) {
        v = table[t];
	while(v) {
	    if(posnear(v->pos,p)) {
		p[0] = v->pos[0];
		p[1] = v->pos[1];
		p[2] = v->pos[2];
		return;
	    }
	    v = v->next;
	}
    }
    t = (TABSIZE-1)*(p[1]-ymin)/deltay;
    v = (vert *)malloc(sizeof(vert));
    v->next = table[t];
    v->pos[0] = p[0];
    v->pos[1] = p[1];
    v->pos[2] = p[2];
    table[t] = v;
}

possame(p1,p2)
float p1[3], p2[3];
{
    if(p1[0]!=p2[0])
	return 0;
    if(p1[1]!=p2[1])
	return 0;
    if(p1[2]!=p2[2])
	return 0;
    return 1;
}

posnear(p1,p2)
float p1[3], p2[3];
{
    float dx, dy, dz;

    dx = p1[0]-p2[0];
    dy = p1[1]-p2[1];
    dz = p1[2]-p2[2];
    if((dx*dx+dy*dy+dz*dz)<(TOLERANCE*TOLERANCE))
	return 1;
    else 
	return 0;
}
