/*
 * Copyright 1991, 1992, 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
/*
 *	tltops -
 *		Draw a triangle list.
 *
 *			Paul Haeberli - 1990
 */
#include "math.h"
#include "stdio.h"
#include "vect.h"
#include "trilist.h"
#include "lum.h"

#define TOLERANCE 0.02
#define GAMMAWARP 0.2

main(argc,argv)
int argc;
char **argv;
{
    FILE *outf;
    int blk;
    float bscale;

    bscale = 1.0;
    if(argc<3) {
	fprintf(stderr, "usage: tltops input.tl out.ps [-b]\n");
	exit(1);
    }
    if(argc>3)
	blk = 1;
    else
	blk = 0;
    outf = fopen(argv[2],"w");
    if(!outf) {
	fprintf(stderr, "tltops: can't open output file %s\n",argv[2]);
	exit(1);
    }
    trilisttops(argv[1],outf,blk,bscale);
}

float greyval(c,bscale)
long c;
float bscale;
{
    float r, g, b;

    r = ((c>>0)&0xff)/255.0;
    g = ((c>>8)&0xff)/255.0;
    b = ((c>>16)&0xff)/255.0;
    r = bscale*LUM(r,g,b);
    if(r>0.99)
	return 0.99;
    else
	return r;
}

cpacktov(c,v)
long c;
vect *v;
{
   v->x = ((c>>0)&0xff)/255.0;
   v->y = ((c>>8)&0xff)/255.0;
   v->z = ((c>>16)&0xff)/255.0;
}

trilisttops(name,outf,blk,bscale)
char *name;
FILE *outf;
int blk;
float bscale;
{
    int i, ntri;
    trilisttri tri;
    vect p0, p1, p2;
    vect c0, c1, c2;

    ntri = tlopen(name);
    fprintf(stderr, "ntri: %d\n",ntri);

    beginps(outf,3.0/2.0,-1.0,1.0,-1.0,1.0);
    for(i=0; i<ntri; i++) {
	tlread(&tri);
	p0.x = tri.x0;
	p0.y = tri.y0;
	p1.x = tri.x1;
	p1.y = tri.y1;
	p2.x = tri.x2;
	p2.y = tri.y2;
#ifdef PUTRGB
	cpacktov(tri.c0,&c0);
	cpacktov(tri.c1,&c1);
	cpacktov(tri.c2,&c2);
	pssubdivtriangle(outf,&p0,&p1,&p2,&c0,&c1,&c2,TOLERANCE);
#else
	p0.z = greyval(tri.c0,bscale);
	p1.z = greyval(tri.c1,bscale);
	p2.z = greyval(tri.c2,bscale);
	if(blk) {
	    p0.z = pow(p0.z,GAMMAWARP);
	    p1.z = pow(p1.z,GAMMAWARP);
	    p2.z = pow(p2.z,GAMMAWARP);
	    cpstriangle(outf,&p0,&p1,&p2,1.0);
	} else {
	    cpstriangle(outf,&p0,&p1,&p2,TOLERANCE);
	}
#endif
    }
    if(blk)
	fprintf(outf,"grestore\n");
    else
	endps();
    tlclose();
}
