/*
 * Copyright 1991, 1992, 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
/* 
 *	warpgeom -
 *		Warp object geometry and normals.
 *
 *			Paul Haeberli - 1990
 */
#include "math.h"
#include "stdio.h"
#include "vect.h"
#include "sgiobj.h"

static warpvert();

float fnoise3();
double pnoise3();
float freq, mag;
int warptype;

#define WARP_SINE	(1)
#define WARP_NOISE	(2)
#define WARP_FLIP	(3)

float mynoise(offset,freq,p)
float offset, freq;
vect *p;
{
    float val;
    vect np;

#define PNOISE
#ifdef PNOISE
    np.x = p->x+offset;
    np.y = p->y+offset;
    np.z = p->z+offset;
    val = pnoise(&np);
#else
    val = fnoise3(freq*(p->x+offset),
		   freq*(p->y-offset),
		   freq*(p->z-offset));
#endif
    return val;
}

/*
 *	This function defines the deformation 
 *
 */
mappoint(p,mp)
vect *p, *mp;
{
    float c, s;

    switch(warptype) {
	case WARP_SINE:
	    s = sin(mag*p->x);
	    c = cos(mag*p->x);
	    mp->x = p->x;
	    mp->y = c*p->y + s*p->z;
	    mp->z = c*p->z - s*p->y;
	    break;
	case WARP_NOISE:
	    mp->x = p->x + mag*mynoise(1.0,freq,p);
	    mp->y = p->y + mag*mynoise(2.0,freq,p);
	    mp->z = p->z + mag*mynoise(3.0,freq,p);
	    break;
	case WARP_FLIP:
	    mp->x = p->x;
	    mp->y = -p->z+mag;
	    mp->z = p->y;
	    break;
    }
}

main(argc,argv)
int argc;
char **argv;
{
    sgiobj *obj;

    if(argc<4) {
	fprintf(stderr,"usage: warpgeom in.sgo out.sgo freq mag\n");
	exit(1);
    }
    warptype = WARP_FLIP;
    obj = readsgiobj(argv[1]);
    freq = atof(argv[3]);
    mag = atof(argv[4]);
    warpgeomobj(obj,mappoint);
    writesgiobj(argv[2],obj);
    exit(0);
}

vabs(v,absv)
vect *v, *absv;
{
    if(v->x<0.0)
	absv->x = -v->x;
    else
	absv->x =  v->x;
    if(v->y<0.0)
	absv->y = -v->y;
    else
	absv->y =  v->y;
    if(v->z<0.0)
	absv->z = -v->z;
    else
	absv->z =  v->z;
}

/*
 *    	generic warpgeomobj follows
 *
 */
#define DELTAMAG	(0.001)

static int (*mapfunc)();

static warpfunc(v)
float *v;
{
    warpvert(v+OFFSET_POINT,v+OFFSET_NORMAL);
}

warpgeomobj(obj,func)
sgiobj *obj;
int (*func)();
{
    mapfunc = func;
    applytoverts(obj,warpfunc);
}

static warpvert(p,n)
vect *p,*n;
{
    vect mp0, mp1, mp2;
    vect p1, p2, mn, c0;
    vect absn;
    float mag; 

    vabs(n,&absn);
    if(absn.x>=absn.y) {
        if(absn.x>=absn.z) {
	    c0.x = 0.0;		/* x biggest */
	    c0.y = 1.0;
	    c0.z = 0.0;
	} else {
	    c0.x = 0.0;		/* z biggest */
	    c0.y = 1.0;
	    c0.z = 0.0;
	}
    } else {
        if(absn.y>=absn.z) {
	    c0.x = 1.0;		/* y biggest */
	    c0.y = 0.0;
	    c0.z = 0.0;
	} else {
	    c0.x = 1.0;		/* z biggest */
	    c0.y = 0.0;
	    c0.z = 0.0;
	}
    }
    vcross(n,&c0,&p1);		/* form first perpendicular */
    vcross(n,&p1,&p2);		/* form second perpendicular */

    mag = vlength(&p1);		/* force to DELTAMAG in length */
    vscale(&p1,DELTAMAG/mag);
    mag = vlength(&p2);
    vscale(&p2,DELTAMAG/mag);

    vadd(p,&p1,&p1);		/* get our sample points */
    vadd(p,&p2,&p2);

    (mapfunc)(p,&mp0);		/* transform all three points */
    (mapfunc)(&p1,&mp1);
    (mapfunc)(&p2,&mp2);

    vsub(&mp1,&mp0,&mp1);	/* get transformed vectors */
    vsub(&mp2,&mp0,&mp2);

    vcross(&mp1,&mp2,&mn);	/* calculate new normal */
    vnormal(&mn);		/* normalize */

    p->x = mp0.x;
    p->y = mp0.y;
    p->z = mp0.z;
    n->x = mn.x;
    n->y = mn.y;
    n->z = mn.z;
}
